//	*****************************************************************************
//
//	Copyright (c) 1998 by David Cole -- All rights reserved.
//
//	Filename:	MFCUTILS.CPP
//
//	Author:		David P. Cole
//
//	Comments:
//
//				This source file may be used in any software for any purpose
//				as long as this notice and the copyright information remain
//				intact.
//
//				Send comments to:
//					David Cole
//					P.O. Box 2813
//					Seattle, WA 98111-2813
//					DLRdave@aol.com
//					http://members.aol.com/DLRdave
//
//	This header block generated by Commentizer v1.0
//
//	*****************************************************************************
//
//	Purpose:
//
//		Utility functions to help with window visibility,
//		document finding and deferred updating.
//
//	History:
//
//		Initial Version - Released with TIA 1.0.0001, November, 1998.
//
//	*****************************************************************************

#include "stdafx.h"
#include "MFCUtils.h"

#include <afxpriv.h>	//	WM_KICKIDLE
//#define WM_KICKIDLE         0x036A  // (params unused) causes idles to kick in


//	***************************************************************************
//	MainWnd stuff:

BOOL MU_GetMainWndVisible(void)
{
	BOOL bVisible= FALSE;
	CWnd *pWnd= AfxGetApp()->GetMainWnd();

	if (NULL!=pWnd)
	{
		bVisible= pWnd->IsWindowVisible();
	}

	return bVisible;
}

void MU_SetMainWndVisible(const BOOL bNewValue /*= TRUE*/)
{
	CWnd *pWnd= AfxGetApp()->GetMainWnd();

	if (NULL!=pWnd)
	{
		if (bNewValue && !pWnd->IsWindowVisible())
		{
			//	Show the main window. Use some logical
			//	criteria for determining if it should be
			//	maximized when shown. Here we simply
			//	maximize it if it's the first time
			//	through this code.
			static BOOL bFirstTime= TRUE;
			BOOL bMax= bFirstTime;
			int nCmdShow= bMax ? SW_SHOWMAXIMIZED : SW_SHOW;
			bFirstTime= FALSE;

			pWnd->ShowWindow(nCmdShow);
			pWnd->BringWindowToTop();
			pWnd->UpdateWindow();
		}
		else if (!bNewValue && pWnd->IsWindowVisible())
		{
			//	Hide the main window, using the same
			//	cleanup routine that MFC does before
			//	calling CloseAllDocuments.
			//	(See MFC source code in APPUI.CPP / WINFRM.CPP)
			AfxGetApp()->HideApplication();
		}
		//	Either of the other two possible
		//	else conditions are no-ops...
	}
}


//	***************************************************************************
//	Document index-based helpers:

int MU_GetCDocumentCount(void)
{
	return AfxGetApp()->GetOpenDocumentCount();
}

CDocument *MU_GetCDocument(const int nIndex)
{
	//	Instead of looping through here blindly, we could check if
	//	our caller is looping through one at a time either forward
	//	or backward. If so, use the last known pos values to
	//	start at a given spot rather than going through all the
	//	GetNext's that we'd have to otherwise:

	//	davidc::fix - Optimization left as an exercise for the reader...

//	Statics which may be useful to implement such an optimization strategy:
	//static int nIndex_Last= 0L;
	//static int nCount_Last= 0L;
	//static POSITION posDT_Last= NULL;
	//static POSITION pos_Last= NULL;

	CDocTemplate *pDT= NULL;
	POSITION posDT= NULL;
	CDocument *pDoc= NULL;
	POSITION pos= NULL;
	int i= 0;

	//	First do the trivial case where we can't get one anyway:
	if (nIndex<1 || nIndex>MU_GetCDocumentCount())
	{
		TRACE(_T("* MU_GetCDocument FAILING : index out of range\015\012"));
		return NULL;
	}

	//	Iterate through app's docs - done by iterating the
	//	app's CDocTemplates and then in turn iterating each
	//	one's CDocuments.
	//	Look for the nIndex'th CDocument:
	posDT= AfxGetApp()->GetFirstDocTemplatePosition();
	pDT= (NULL==posDT) ? NULL : AfxGetApp()->GetNextDocTemplate(posDT);
	while (NULL!=pDT)
	{
		pos= pDT->GetFirstDocPosition();
		pDoc= (NULL==pos) ? NULL : pDT->GetNextDoc(pos);
		while (NULL!=pDoc)
		{
			i++;
			if (i==nIndex)
			{
				//	Found it! pDoc points to the nIndex'th document:
				break;
			}

			pDoc= (NULL==pos) ? NULL : pDT->GetNextDoc(pos);
		}

		pDT= (NULL==posDT) ? NULL : AfxGetApp()->GetNextDocTemplate(posDT);
	}

	if (NULL==pDoc)
	{
		TRACE(_T("* MU_GetCDocument returning NULL : iteration failed?\015\012"));
	}

	return pDoc;
}

LPUNKNOWN MU_GetCDocumentInterface(const int nIndex, const IID &iid)
{
	//	If non-NULL, returned LPUNKNOWN must be Release-d by caller:
	HRESULT hr= NOERROR;
	LPUNKNOWN ipUnk= NULL;
	CDocument *pDoc= NULL;

	//	Try to get the CDocument first:
	pDoc= MU_GetCDocument(nIndex);

	if (NULL==pDoc)
	{
		//	Bail if not found:
		return NULL;
	}

	//	If we got the doc, QueryInterface for the requested interface pointer:
	if (NULL!=pDoc)
	{
		hr= (HRESULT) pDoc->ExternalQueryInterface(&iid, (void **) &ipUnk);
		if (FAILED(hr))
		{
			AfxThrowOleException(hr);
		}
	}

	//	If non-NULL, returned LPUNKNOWN must be Release-d by caller:
	return ipUnk;
}

LPDISPATCH MU_GetCDocumentDispatch(const int nIndex)
{
	//	If non-NULL, returned LPDISPATCH must be Release-d by caller:
	return (LPDISPATCH) MU_GetCDocumentInterface(nIndex, IID_IDispatch);
}


//	***************************************************************************
//	Document name-based helpers:

CDocument *MU_FindCDocument(LPCTSTR szName, const long nCompare /*= MU_CMP_PATHNAME*/)
{
	CDocTemplate *pDT= NULL;
	POSITION posDT= NULL;
	CDocument *pDoc= NULL;
	POSITION pos= NULL;
	CString cstrName;

	//	First do the trivial case where we can't find one anyway:
	if (NULL==szName || TCHAR(0)==szName[0])
	{
		TRACE(_T("* MU_FindCDocument returning NULL : szName is empty or NULL\015\012"));
		return NULL;
	}

	ASSERT(MU_CMP_PATHNAME==nCompare || MU_CMP_TITLE==nCompare);
		//	Anything else is an unsupported value for nCompare
		//	and will definitely not find any documents...

	cstrName= szName;

	//	Iterate through app's docs - done by iterating the
	//	app's CDocTemplates and then in turn iterating each
	//	one's CDocuments.
	//	Look for the CDocument that has cstrName as its
	//	PathName or Title:
	posDT= AfxGetApp()->GetFirstDocTemplatePosition();
	pDT= (NULL==posDT) ? NULL : AfxGetApp()->GetNextDocTemplate(posDT);
	while (NULL!=pDT)
	{
		pos= pDT->GetFirstDocPosition();
		pDoc= (NULL==pos) ? NULL : pDT->GetNextDoc(pos);
		while (NULL!=pDoc)
		{
			if ( (MU_CMP_PATHNAME==nCompare && 0==cstrName.CompareNoCase(pDoc->GetPathName()))
				 || (MU_CMP_TITLE==nCompare && 0==cstrName.CompareNoCase(pDoc->GetTitle())) )
			{
				break;	//	found it!
			}

			pDoc= (NULL==pos) ? NULL : pDT->GetNextDoc(pos);
		}

		pDT= (NULL==posDT) ? NULL : AfxGetApp()->GetNextDocTemplate(posDT);
	}

	if (NULL==pDoc)
	{
		TRACE(_T("* MU_FindCDocument returning NULL : szName NOT FOUND\015\012"));
	}

	return pDoc;
}

LPUNKNOWN MU_FindCDocumentInterface(LPCTSTR szName, const IID &iid)
{
	//	If non-NULL, returned LPUNKNOWN must be Release-d by caller:
	HRESULT hr= NOERROR;
	LPUNKNOWN ipUnk= NULL;
	CDocument *pDoc= NULL;

	//	Try to find the CDocument first by PathName:
	pDoc= MU_FindCDocument(szName, MU_CMP_PATHNAME);

	if (NULL==pDoc)
	{
		//	Then by Title if PathName didn't find it:
		pDoc= MU_FindCDocument(szName, MU_CMP_TITLE);
	}

	if (NULL==pDoc)
	{
		//	Bail if not found:
		return NULL;
	}

	//	If we got the doc, QueryInterface for the requested interface pointer:
	if (NULL!=pDoc)
	{
		hr= (HRESULT) pDoc->ExternalQueryInterface(&iid, (void **) &ipUnk);
		if (FAILED(hr))
		{
			AfxThrowOleException(hr);
		}
	}

	//	If non-NULL, returned LPUNKNOWN must be Release-d by caller:
	return ipUnk;
}

LPDISPATCH MU_FindCDocumentDispatch(LPCTSTR szName)
{
	//	If non-NULL, returned LPDISPATCH must be Release-d by caller:
	return (LPDISPATCH) MU_FindCDocumentInterface(szName, IID_IDispatch);
}


//	***************************************************************************
//	Document VARIANT-based helpers
//		(forward on to index-based or name-based helpers
//		 according to VARIANT type or changed-to type)

CDocument *MU_GetCDocumentFromVariant(const VARIANT &vDocID)
{
	HRESULT hr= NOERROR;
	TCHAR szName[2*MAX_PATH]= _T("");
	VARIANT vLocal;
	CDocument *pDoc= NULL;

	//	The string case is the primary case. No matter what happens in
	//	here, the function returns at the closing brace of the VT_BSTR
	//	case. This is so that other cases can be converted to VT_BSTR
	//	if possible and then call this function recursively. The return
	//	at the closing brace of the VT_BSTR case guarantees that we
	//	will not recurse infinitely, converting back and forth between
	//	two types.

	if (VT_BSTR==V_VT(&vDocID))
	{
		wsprintf(szName, _T("%ls"), V_BSTR(&vDocID));

		//	Try to find the CDocument first by PathName:
		pDoc= MU_FindCDocument(szName, MU_CMP_PATHNAME);
		if (NULL==pDoc)
		{
			//	Then by Title if PathName didn't find it:
			pDoc= MU_FindCDocument(szName, MU_CMP_TITLE);
		}

		if (NULL==pDoc)
		{
			//	If we couldn't find the named document, then perhaps
			//	the passed in VARIANT is a string that really represents
			//	a number: the index between 1 and n in our collection.

			//	Try converting the variant to an I4:
			VariantInit(&vLocal);
			hr= VariantChangeType(&vLocal, (VARIANT *) &vDocID, 0, VT_I4);
			if (SUCCEEDED(hr))
			{
				pDoc= MU_GetCDocument(V_I4(&vLocal));
			}
			VariantClear(&vLocal);
		}

		//	This is the end of the line; if we haven't found it by name
		//	or the name wasn't a string representation of the numeric
		//	index, then the document identified by vDocID simply does
		//	not exist!

		return pDoc;
	}
	else if (VT_I4==V_VT(&vDocID))
	{
		//	Try using vDocID as an index into our 1 to n collection:
		pDoc= MU_GetCDocument(V_I4(&vDocID));
	}
	else if ( (VT_DISPATCH|VT_BYREF)==V_VT(&vDocID) || (VT_UNKNOWN|VT_BYREF)==V_VT(&vDocID)
		|| VT_DISPATCH==V_VT(&vDocID) || VT_UNKNOWN==V_VT(&vDocID) )
	{
		//	Maybe it's an interface onto the CDocument directly.
		//	If so, get the associated CDocument by querying for IDispatch
		//	and calling CCmdTarget::FromIDispatch.

		LPUNKNOWN pUnk= NULL;
		LPDISPATCH pDisp= NULL;

		if ((VT_DISPATCH|VT_BYREF)==V_VT(&vDocID))
		{
			if (NULL!=V_DISPATCHREF(&vDocID))
			{
				pUnk= * (V_DISPATCHREF(&vDocID));
			}
		}
		else if ((VT_UNKNOWN|VT_BYREF)==V_VT(&vDocID))
		{
			if (NULL!=V_UNKNOWNREF(&vDocID))
			{
				pUnk= * (V_UNKNOWNREF(&vDocID));
			}
		}
		else if (VT_DISPATCH==V_VT(&vDocID))
		{
			pUnk= V_DISPATCH(&vDocID);
		}
		else if (VT_UNKNOWN==V_VT(&vDocID))
		{
			pUnk= V_UNKNOWN(&vDocID);
		}

		if (NULL!=pUnk)
		{
			hr= pUnk->QueryInterface(IID_IDispatch, (void **) &pDisp);
			if (SUCCEEDED(hr))
			{
				pDoc= (CDocument *) CCmdTarget::FromIDispatch(pDisp);
				if (NULL!=pDoc && !pDoc->IsKindOf(RUNTIME_CLASS(CDocument)))
				{
					//	False alarm, this IDispatch is *NOT* associated with a CDocument:
					pDoc= NULL;
				}

				pDisp->Release();
				pDisp= NULL;
			}
		}
	}


	//	If one of the cases above did NOT find a document, or
	//	if the type of the VARIANT vDocID is not covered by
	//	one of the cases above, try converting the VARIANT
	//	into a BSTR and calling this function again. The return
	//	at the closing brace of the VT_BSTR if above guarantees
	//	that we won't try to recurse indefinitely.

	if (NULL==pDoc)
	{
		VariantInit(&vLocal);
		hr= VariantChangeType(&vLocal, (VARIANT *) &vDocID, 0, VT_BSTR);
		if (SUCCEEDED(hr))
		{
			pDoc= MU_GetCDocumentFromVariant(vLocal);
		}
		VariantClear(&vLocal);
	}


	return pDoc;
}

LPUNKNOWN MU_GetCDocumentInterfaceFromVariant(const VARIANT &vDocID, const IID &iid)
{
	//	If non-NULL, returned LPUNKNOWN must be Release-d by caller:
	HRESULT hr= NOERROR;
	LPUNKNOWN ipUnk= NULL;
	CDocument *pDoc= NULL;

	//	Try to get the CDocument first:
	pDoc= MU_GetCDocumentFromVariant(vDocID);

	if (NULL==pDoc)
	{
		//	Bail if not found:
		return NULL;
	}

	//	If we got the doc, QueryInterface for the requested interface pointer:
	if (NULL!=pDoc)
	{
		hr= (HRESULT) pDoc->ExternalQueryInterface(&iid, (void **) &ipUnk);
		if (FAILED(hr))
		{
			AfxThrowOleException(hr);
		}
	}

	//	If non-NULL, returned LPUNKNOWN must be Release-d by caller:
	return ipUnk;
}

LPDISPATCH MU_GetCDocumentDispatchFromVariant(const VARIANT &vDocID)
{
	//	If non-NULL, returned LPDISPATCH must be Release-d by caller:
	return (LPDISPATCH) MU_GetCDocumentInterfaceFromVariant(vDocID, IID_IDispatch);
}


//	***************************************************************************
//	DeferringUpdates flag:

BOOL bDeferring= FALSE;
DWORD dwLastTimeDeferSet= 0;

void MU_SetDeferringUpdates(BOOL bNewValue)
{
	bDeferring= bNewValue;

	if (bDeferring)
	{
#ifdef _DEBUG
		DWORD dwPrev= dwLastTimeDeferSet;
#endif

		dwLastTimeDeferSet= GetTickCount();

#ifdef _DEBUG
		TCHAR sz[256]= _T("");
		wsprintf(sz, _T("* MU_SetDeferringUpdates : Elapsed ticks since last call : %4ld\015\012"), (long)(dwLastTimeDeferSet - dwPrev));
		TRACE(sz);
#endif
	}
	else
	{
		TRACE(_T("* MU_SetDeferringUpdates : Deferring is OFF.\015\012"));
	}
}

BOOL MU_StopDeferringUpdatesAfter(DWORD dwTime /*= 1500*/)
{
	BOOL bStopped= FALSE;

	//	If enough time has passed since SetDeferringUpdates(TRUE)
	//	was called, then SetDeferringUpdates(FALSE) can be called.

	if ((GetTickCount()-dwLastTimeDeferSet > dwTime) && MU_GetDeferringUpdates())
	{
		TRACE(_T("*\015\012"));
		TRACE(_T("* MU_StopDeferringUpdatesAfter STOPPED Deferring...\015\012"));

		MU_SetDeferringUpdates(FALSE);
		bStopped= TRUE;

		//	Now that we've stopped deferring updates, let's
		//	kick one more idle message through with a PostMessage
		//	so that our documents get some idle time with
		//	MU_GetDeferringUpdates returning FALSE. This way,
		//	if any of them were Changed during a deferral period,
		//	their OnIdle will detect that, re-transmit the Changed
		//	message and the corresponding Views and Items can
		//	finally be updated now that deferrals are off.
		CWnd *pWnd= AfxGetApp()->GetMainWnd();
		if (NULL!=pWnd)
		{
			pWnd->PostMessage(WM_KICKIDLE);
		}
	}

	return bStopped;
}

BOOL MU_GetDeferringUpdates(void)
{
	return bDeferring;
}
