//	*****************************************************************************
//
//	Copyright (c) 1998 by David Cole -- All rights reserved.
//
//	Filename:	TIADOC.CPP
//
//	Author:		David P. Cole
//
//	Comments:
//
//				This source file may be used in any software for any purpose
//				as long as this notice and the copyright information remain
//				intact.
//
//				Send comments to:
//					David Cole
//					P.O. Box 2813
//					Seattle, WA 98111-2813
//					DLRdave@aol.com
//					http://members.aol.com/DLRdave
//
//	This header block generated by Commentizer v1.0
//
//	*****************************************************************************
//
//	Purpose:
//
//		Implementation of the CTIADoc object.
//			Important points about the code in this file:
//			- The document owns the list of items (rectangles)
//				and owns each item individually. The document
//				alone has responsibility for creating and
//				deleting these items as they're needed.
//			- After calling the ForceClose method, no further
//				references to the document should be made except
//				through Automation methods. Automation methods
//				should check for IsClosed() if they cannot
//				operate properly with a closed document.
//
//	History:
//
//		Initial Version - Released with TIA 1.0.0001, November, 1998.
//
//	*****************************************************************************

// TIADoc.cpp : implementation of the CTIADoc class
//

#include "stdafx.h"
#include "TIA.h"

#include "TIADoc.h"
#include "SrvrItem.h"

#include "TIAItems.h"
#include "MFCUtils.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTIADoc

IMPLEMENT_DYNCREATE(CTIADoc, COleServerDoc)

BEGIN_MESSAGE_MAP(CTIADoc, COleServerDoc)
	//{{AFX_MSG_MAP(CTIADoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CTIADoc, COleServerDoc)
	//{{AFX_DISPATCH_MAP(CTIADoc)
	DISP_PROPERTY_EX(CTIADoc, "Name", GetName, SetNotSupported, VT_BSTR)
	DISP_PROPERTY_EX(CTIADoc, "FullPathName", GetFullPathName, SetNotSupported, VT_BSTR)
	DISP_PROPERTY_EX(CTIADoc, "Modified", GetModified, SetModified, VT_I4)
	DISP_PROPERTY_EX(CTIADoc, "Visible", GetVisible, SetVisible, VT_I4)
	DISP_FUNCTION(CTIADoc, "Save", Save, VT_I4, VTS_NONE)
	DISP_FUNCTION(CTIADoc, "SaveAs", SaveAs, VT_I4, VTS_BSTR)
	DISP_FUNCTION(CTIADoc, "AddColoredRect", AddColoredRect, VT_DISPATCH, VTS_I4 VTS_I4 VTS_I4 VTS_I4 VTS_I4)
	DISP_DEFVALUE(CTIADoc, "Name")
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// *****************************************************************************
//	ClassWizard Leftovers:

// Note: we add support for IID_ITIA to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {091779A2-5E25-11D2-9B8C-44455354616F}
//static const IID IID_ITIA =
//{ 0x91779a2, 0x5e25, 0x11d2, { 0x9b, 0x8c, 0x44, 0x45, 0x53, 0x54, 0x61, 0x6f } };
// *****************************************************************************
//	CHANGED_GUID

//	Since we include the generated header file, TiaTLB.h,
//	use the ID's defined in there:

BEGIN_INTERFACE_MAP(CTIADoc, COleServerDoc)
	INTERFACE_PART(CTIADoc, DIID_ITIADocumentDisp, Dispatch)		//	CHANGED_GUID
	INTERFACE_PART(CTIADoc, IID_ITIADocument, Dual_TIA_Doc)			//  DUAL_INTERFACE_SUPPORT
	DUAL_ERRORINFO_PART(CTIADoc)									//  DUAL_INTERFACE_SUPPORT
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTIADoc construction/destruction

CTIADoc::CTIADoc()
{
	// Use OLE compound files
//	EnableCompoundFile();

	EnableAutomation();

	AfxOleLockApp();

	m_bChangedWhileDeferring= FALSE;
	m_dwAutomationFlags= AF_NONE;
}

CTIADoc::~CTIADoc()
{
	DeleteTIAItems();
	AfxOleUnlockApp();
}

BOOL CTIADoc::OnNewDocument()
{
	if (!COleServerDoc::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CTIADoc server implementation

COleServerItem* CTIADoc::OnGetEmbeddedItem()
{
	// OnGetEmbeddedItem is called by the framework to get the COleServerItem
	//  that is associated with the document.  It is only called when necessary.

	CTIASrvrItem* pItem = new CTIASrvrItem(this);
	ASSERT_VALID(pItem);
	return pItem;
}

/////////////////////////////////////////////////////////////////////////////
// CTIADoc serialization

void CTIADoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
		m_TIADocItems.Serialize(ar);
	}
	else
	{
		// TODO: add loading code here
		ASSERT(m_TIADocItems.IsEmpty());
		m_TIADocItems.Serialize(ar);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CTIADoc diagnostics

#ifdef _DEBUG
void CTIADoc::AssertValid() const
{
	COleServerDoc::AssertValid();
}

void CTIADoc::Dump(CDumpContext& dc) const
{
	COleServerDoc::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTIADoc commands


//	***************************************************************************
//	Virtual method overrides:

void CTIADoc::OnFinalRelease() 
{
	//	TIA - Added to handle deferring updates:
	//	(If we got modified while updates were being
	//	deferred, but now updates are NOT being deferred,
	//	then send the change again so that updates can
	//	now take place...)
	if (m_bChangedWhileDeferring && !MU_GetDeferringUpdates())
	{
		CRect r(0, 0, 0, 0);
		Changed(r);
	}

	COleServerDoc::OnFinalRelease();
}


void CTIADoc::OnIdle() 
{
	//	Only call the parent (which resolves to COleDocument::OnIdle)
	//	if the app's *MAIN* frame window is visible. The parent class
	//	method decides whether or not to call OnShowViews based on the
	//	visibility bit of the MFCChildFrame window. We only want that
	//	to be called if the app's main window is also visible. If called
	//	when main frame is invisible it puts a lock on the Doc object and
	//	even though all external references to it go away, we have a ref
	//	count of 2 left on the Doc.

	//	The following code is an exact copy of COleDocument::OnIdle from
	//	MFC\SRC\OLEDOC1.CPP except for the lines marked with LOCAL_CHANGE

	ASSERT_VALID(this);

	// determine if any visible views are on this document
	BOOL bVisible = FALSE;

	//	A view can only be "visible" in this sense of locking		LOCAL_CHANGE
	//	the doc if the app's main wnd is visible too...				LOCAL_CHANGE
	if (MU_GetMainWndVisible())									//	LOCAL_CHANGE
	{															//	LOCAL_CHANGE
		POSITION pos = GetFirstViewPosition();
		while (pos != NULL)
		{
			CView* pView = GetNextView(pos);
			ASSERT_VALID(pView);
			CFrameWnd* pFrameWnd = pView->GetParentFrame();
			ASSERT_VALID(pFrameWnd);
			if (pFrameWnd->GetStyle() & WS_VISIBLE)
			{
				bVisible = TRUE;
				break;
			}
		}
	}															//	LOCAL_CHANGE

	// when state has changed, call OnShowViews
	if (bVisible != m_bLastVisible)
	{
		OnShowViews(bVisible);
		m_bLastVisible = bVisible;
	}


	//	TIA - Added to handle deferring updates:
	//	(If we got modified while updates were being
	//	deferred, but now updates are NOT being deferred,
	//	then send the change again so that updates can
	//	now take place...)
	if (m_bChangedWhileDeferring && !MU_GetDeferringUpdates())
	{
		TRACE(_T("*\015\012"));
		TRACE(_T("* CTIADoc::OnIdle re-transmitting deferred Changed...\015\012"));

		CRect r(0, 0, 0, 0);
		Changed(r);
	}
}


//	***************************************************************************
//	Call this whenever something about the documents contents changes.
//	The rect should indicate the area of additional document content.
//	Pass in a CRect(0,0,0,0) if change is a modification or deletion.

HRESULT CTIADoc::Changed(const CRect & /*r*/)
{
	if (IsClosed())
	{
		return NOERROR;
	}

	SetModifiedFlag(TRUE);

	//	To avoid constant redraws during automation loops:
	if (MU_GetDeferringUpdates())
	{
		m_bChangedWhileDeferring= TRUE;
	}
	else
	{
		m_bChangedWhileDeferring= FALSE;
		//UpdateDocRect(r);
		UpdateAllItems(NULL);	//	CTIASrvrItem - for embeds in containers
		UpdateAllViews(NULL);
	}

	return NOERROR;
}


//	***************************************************************************
//	TIA_DocItem related stuff:

HRESULT CTIADoc::AddTIARect(const CRect &r, const COLORREF &rgb, TIA_ColoredRectangle **ppRect /*= NULL*/)
{
	TIA_ColoredRectangle *pTIARect= NULL;

	//	Make a new item and add it to our list:
	pTIARect= new TIA_ColoredRectangle;
	pTIARect->SetDoc(this);
	pTIARect->SetRect(r);
	pTIARect->SetColor(rgb);

	m_TIADocItems.AddTail(pTIARect);

	Changed(r);

	//	Set output arg:
	if (NULL!=ppRect)
	{
		(*ppRect)= pTIARect;
	}

	return NOERROR;
}

HRESULT CTIADoc::DeleteTIAItems(void)
{
	TIA_DocItem *pItem= NULL;
	while (!m_TIADocItems.IsEmpty())
	{
		pItem= (TIA_DocItem *) m_TIADocItems.RemoveTail();
		delete pItem;
		pItem= NULL;
	}

	return NOERROR;
}

HRESULT CTIADoc::GetTIAItemList(CObList **ppList)
{
	(*ppList)= &m_TIADocItems;
	return NOERROR;
}


//	***************************************************************************
//	Automation methods:

BSTR CTIADoc::GetName() 
{
	CString strResult(GetTitle());
	return strResult.AllocSysString();
}

BSTR CTIADoc::GetFullPathName() 
{
	CString strResult(GetPathName());
	return strResult.AllocSysString();
}


BOOL CTIADoc::DoSave(LPCTSTR lpszPathName, BOOL bReplace)
	// Save the document data to a file
	// lpszPathName = path name where to save document file
	// if lpszPathName is NULL then the user will be prompted (SaveAs)
	// note: lpszPathName can be different than 'm_strPathName'
	// if 'bReplace' is TRUE will change file name if successful (SaveAs)
	// if 'bReplace' is FALSE will not change path name (SaveCopyAs)
{
	CString newName = lpszPathName;
	if (newName.IsEmpty())
	{
		CDocTemplate* pTemplate = GetDocTemplate();
		ASSERT(pTemplate != NULL);

		newName = m_strPathName;
		if (bReplace && newName.IsEmpty())
		{
			newName = m_strTitle;
#ifndef _MAC
			// check for dubious filename
			int iBad = newName.FindOneOf(_T(" #%;/\\"));
#else
			int iBad = newName.FindOneOf(_T(":"));
#endif
			if (iBad != -1)
				newName.ReleaseBuffer(iBad);

#ifndef _MAC
			// append the default suffix if there is one
			CString strExt;
			if (pTemplate->GetDocString(strExt, CDocTemplate::filterExt) &&
			  !strExt.IsEmpty())
			{
				ASSERT(strExt[0] == '.');
				newName += strExt;
			}
#endif
		}

		if (!AfxGetApp()->DoPromptFileName(newName,
		  bReplace ? AFX_IDS_SAVEFILE : AFX_IDS_SAVEFILECOPY,
		  OFN_HIDEREADONLY | OFN_PATHMUSTEXIST, FALSE, pTemplate))
			return FALSE;       // don't even attempt to save
	}

	CWaitCursor wait;

	if (!OnSaveDocument(newName))
	{
		if (lpszPathName == NULL)
		{
			// be sure to delete the file
			TRY
			{
				CFile::Remove(newName);
			}
			CATCH_ALL(e)
			{
				TRACE0("Warning: failed to delete file after failed SaveAs.\n");
				//DELETE_EXCEPTION(e);
			}
			END_CATCH_ALL
		}
		return FALSE;
	}

	// reset the title and change the document name
	if (bReplace)
		SetPathName(newName);

	return TRUE;        // success
}


//	If 0xFFFFFFFFL==dwAttrib, then GetFileAttributes failed.
//	It most likely means that the file doesn't exist, which
//	is OK in the SaveAs case... We *could* call GetLastError
//	to verify what exactly went wrong with GetFileAttributes.
//	For these automation methods, we'll just attempt the Save/As
//	as long as we're definitely NOT overwriting an existing
//	read-only file.

long CTIADoc::Save() 
{
	//	If this file has never been saved (i.e. there's not
	//	a valid path name) then we can't save via this method
	//	without interacting with the user. We don't want to
	//	do that from an automation method, so just bail with
	//	an "unexpected" failure.

	DWORD dwAttrib= 0;

	if (GetPathName().IsEmpty() || IsClosed() || (NULL!=m_lpClientSite))
	{
		AfxThrowOleException(E_UNEXPECTED);
	}

	dwAttrib= GetFileAttributes(GetPathName());
	if (0xFFFFFFFFL!=dwAttrib && (dwAttrib & FILE_ATTRIBUTE_READONLY))
	{
		//	The file exists, but we do not have read-write access
		AfxThrowOleException(E_FAIL);
	}

	//	Trigger deferred updates before we Save:
	if (m_bChangedWhileDeferring)
	{
		CRect r(0, 0, 0, 0);
		MU_SetDeferringUpdates(FALSE);
		Changed(r);
	}

	return (long) DoSave(GetPathName());
}


long CTIADoc::SaveAs(LPCTSTR szFullPathName) 
{
	DWORD dwAttrib= 0;
	CString cstrFullPathName;
	IPersistFile *pPF= NULL;
	BOOL bRemember= TRUE;
	HRESULT hr= NOERROR;
	BSTR bstrName= NULL;

	//	Could add an optional flags argument to this method which
	//	indicates export to a *.txt file or a *.wmf file... or any
	//	format that you can write!

	//	davidc::fix - what about UNICODE?
	char szResolved[MAX_PATH]= "";
	_fullpath(szResolved, szFullPathName, sizeof(szResolved));
	cstrFullPathName= szResolved;

	if (IsClosed())
	{
		AfxThrowOleException(E_UNEXPECTED);
	}

	dwAttrib= GetFileAttributes(cstrFullPathName);
	if (0xFFFFFFFFL!=dwAttrib && (dwAttrib & FILE_ATTRIBUTE_READONLY))
	{
		//	The file exists, but we do not have read-write access
		AfxThrowOleException(E_FAIL);
	}

	//	If the full path name is NOT *this* document's current
	//	full path name, then check to see if it is the full
	//	path name of another open document. If it is, this
	//	SaveAs attempt must fail, because we can't overwrite
	//	another document's file while it is open.
	if (0!=cstrFullPathName.CompareNoCase(GetPathName()))
	{
		//	This is not our own full path name, see if it
		//	is another open document's full path name:
		CDocument *pOtherDoc= MU_FindCDocument(cstrFullPathName);
		if (NULL!=pOtherDoc)
		{
			//	Yep - there's a document open with that name...
			AfxThrowOleException(STG_E_FILEALREADYEXISTS);
		}
	}

	//	Trigger deferred updates before we Save:
	if (m_bChangedWhileDeferring)
	{
		CRect r(0, 0, 0, 0);
		MU_SetDeferringUpdates(FALSE);
		Changed(r);
	}

	if (NULL!=m_lpClientSite)
	{
		//	We're an embedded item in some container; make this
		//	a "Save Copy As"...
		long nRet= 0L;

		//	The portions between BEGIN / END_SAVECOPYAS are copied
		//	from COleServerDoc::OnFileSaveCopyAs.
		//BEGIN_SAVECOPYAS
			ASSERT_VALID(this);
			ASSERT(m_bRemember);
			//ASSERT(m_lpClientSite != NULL);	//	unnecessary - above if statement guarantees

			LPSTORAGE lpOrigStg = m_lpRootStg;
			m_lpRootStg = NULL; // ignore embedded storage for now

			TRY
			{
				// call DoSave to perform Save Copy As...
				m_bRemember = FALSE;
				nRet= DoSave(cstrFullPathName, FALSE);	//	nRet= added... , NULL changed to szFullPathName
			}
			END_TRY

			m_lpRootStg = lpOrigStg;
			m_bRemember = TRUE;
		//END_SAVECOPYAS

		return nRet;
	}

	return (long) DoSave(cstrFullPathName);
}


LPDISPATCH CTIADoc::AddColoredRect(long nLeft, long nTop, long nWidth, long nHeight, long nRGB) 
{
	HRESULT hr= NOERROR;
	LPDISPATCH ipDisp= NULL;
	TIA_ColoredRectangle *pRect= NULL;

	CRect r(nLeft, nTop, nLeft+nWidth, nTop+nHeight);

	//	When modifying the doc via Automation, defer updates until
	//	they're necessary (avoids unsightly and time-consuming
	//	redraw while looping):
	MU_SetDeferringUpdates();

	hr= AddTIARect(r, nRGB, &pRect);
	if (FAILED(hr))
	{
		AfxThrowOleException(hr);
	}

	ipDisp= pRect->GetAutomationObject();
	if (NULL==ipDisp)
	{
		AfxThrowOleException(E_OUTOFMEMORY);
	}

	return ipDisp;
}

long CTIADoc::GetModified() 
{
	return (long) (IsModified() && !IsClosed());
}

void CTIADoc::SetModified(long nNewValue) 
{
	if (IsClosed())
	{
		AfxThrowOleException(E_UNEXPECTED);
	}

	m_bChangedWhileDeferring= nNewValue ? TRUE : FALSE;
	SetModifiedFlag(m_bChangedWhileDeferring);
}

long CTIADoc::GetVisible() 
{
	BOOL bVisible= FALSE;

	if (MU_GetMainWndVisible() && !IsClosed())
	{
		POSITION pos = GetFirstViewPosition();
		while (pos != NULL)
		{
			CView* pView = GetNextView(pos);
			ASSERT_VALID(pView);
			CFrameWnd* pFrameWnd = pView->GetParentFrame();
			ASSERT_VALID(pFrameWnd);
			if (pFrameWnd->GetStyle() & WS_VISIBLE)
			{
				bVisible = TRUE;
				break;
			}
		}
	}

	return (long) bVisible;
}

void CTIADoc::SetVisible(long nNewValue) 
{
	BOOL bUserCtrl= AfxOleGetUserCtrl();

	if (IsClosed())
	{
		AfxThrowOleException(E_UNEXPECTED);
	}

	AfxOleSetUserCtrl(TRUE);	//	avoids hiding app window inside OnShowDocument

	if (nNewValue && !MU_GetMainWndVisible())
	{
		MU_SetMainWndVisible();
	}

	OnShowDocument(nNewValue ? TRUE : FALSE);

	AfxOleSetUserCtrl(bUserCtrl);
}

void CTIADoc::DeleteContents() 
{
	DeleteTIAItems();
	COleServerDoc::DeleteContents();
}

HRESULT CTIADoc::ForceClose(void)
{
	//	Close all the views onto the document such that
	//	the locks that views have on it will go away.
	//	Then when we get a final release, we'll go
	//	through the actual document destruction process.

	//	Remember, if you call this method from an automation
	//	controller, you're also zombifying the document object
	//	for anyone else who has a programmatic hold on it.
	//		(That is, many methods on the document will not
	//		 work after a Close has been executed...)

	//	It also forces the views onto the document to close
	//	and so disappears from the user's view if the app
	//	is visible. Make sure that users of your program
	//	expect this to happen when it does...!


	if (IsClosed())
	{
		//	Already closed, simply return:
		return NOERROR;
	}


	if (IsInPlaceActive())
	{
		//	Just deactivate and return:
		OnDeactivate();
		ASSERT(NULL==m_pInPlaceFrame);
		return NOERROR;

		//	Don't mark it as closed because it's still valid
		//	while there are references to it when it's an embed.
	}


	if (NULL!=m_lpClientSite)
	{
		//	If we have a client site, do a DoVerb(HIDE)
		//	to force closed any open windows onto the document

		//	Otherwise, we'd be half-mangling ourselves
		//	while we still belong to our client site.

		HRESULT hr= m_xOleObject.DoVerb(OLEIVERB_HIDE, NULL, NULL, -1, NULL, NULL);

		TRACE("* DoVerb(HIDE) from CTIADoc.ForceClose because we're embedded.\n");

		return NOERROR;

		//	Don't mark it as closed because it's still valid
		//	while there are references to it when it's an embed.
	}


	//	Double-check to make sure we're not embedded/in-place active:
	ASSERT(!IsInPlaceActive());
	ASSERT(NULL==m_pInPlaceFrame);
	ASSERT(NULL==m_lpClientSite);


	//	Presumably, an external agent has a reference to us,
	//	or we would not be in an automation method... Still,
	//	to be cautious, we'll add our own ref until we're done
	//	here:
	InternalAddRef();

	//	We are programmatically being called to close the document;
	//	force it to be "clean" so no prompting ever occurs:
	SetModifiedFlag(FALSE);
		//	If data needs to be saved then call Save/As
		//	programmatically BEFORE calling Close - duh!


	//	We CANNOT call OnCloseDocument directly because we
	//	do NOT want CoDisconnectObject to be called. Our
	//	external references are still "valid" and must be
	//	released by their holders before we can really call
	//	OnCloseDocument directly. (The parent class calls
	//	OnCloseDocument during OnFinalRelease...)

	//	So we have to piece together from all of our parent
	//	classes the pieces of OnCloseDocument that we DO
	//	need to call to coerce its internal holders (views
	//	and visible locks) into releasing the doc.


	//	The portions between BEGIN / END_DEACTIVATE are inferred
	//	as necessary from COleServerDoc::OnCloseDocument.
	//		(The inplace active case is taken care of above...)
	//BEGIN_DEACTIVATE
		UpdateVisibleLock(FALSE, FALSE);
	//END_DEACTIVATE

	//	The portions between BEGIN / END_LINKING are inferred
	//	as necessary from COleLinkingDoc::OnCloseDocument.
	//BEGIN_LINKING
		if (m_bVisibleLock)
		{
			m_bVisibleLock= FALSE;
			LockExternal(FALSE, FALSE);
		}
	//END_LINKING

	//	The portions between BEGIN / END_VIEW_CLOSING are copied
	//	from CDocument::OnCloseDocument.
	//BEGIN_VIEW_CLOSING
		// destroy all frames viewing this document
		// the last destroy may destroy us
		BOOL bAutoDelete = m_bAutoDelete;
		m_bAutoDelete = FALSE;  // don't destroy document while closing views
		while (!m_viewList.IsEmpty())
		{
			// get frame attached to the view
			CView* pView = (CView*)m_viewList.GetHead();
			ASSERT_VALID(pView);
			CFrameWnd* pFrame = pView->GetParentFrame();
			ASSERT_VALID(pFrame);

			// and close it
			PreCloseFrame(pFrame);
			pFrame->DestroyWindow();
				// will destroy the view as well
		}
		//	m_bAutoDelete = bAutoDelete;
		m_bAutoDelete = TRUE;		//	force the delete in the OnFinalRelease
	//END_VIEW_CLOSING


	//	Remove the "created with" refcount so that
	//	when external holders release us, our count
	//	goes to 0 and OnFinalRelease is called.


	//	CCmdTargets are created with a refcount of
	//	1; in the case of a document created through
	//	user actions (as opposed to embeddings or
	//	automation actions -- both indicated by the
	//	IsEmbedded() method) the refcount will NEVER
	//	go to 0 unless we force it down by one here
	//	in this Close method...

	if (!IsEmbedded())
	{
		InterlockedDecrement(&m_dwRef);
	}


	MarkAsClosed();

	InternalRelease();		//	to balance our above call

	return NOERROR;
}


// *****************************************************************************
//  DUAL_INTERFACE_SUPPORT

//	In addition to this bottom section of the file, there are also two extra
//	entries in the interface map up above. One for the dual interface and one
//	for ISupportErrorInfo.

//	Delegate standard IDispatch methods to MFC IDispatch implementation:
DELEGATE_DUAL_INTERFACE(CTIADoc, Dual_TIA_Doc)

//	Implement ISupportErrorInfo for this dual interface:
IMPLEMENT_DUAL_ERRORINFO(CTIADoc, IID_ITIADocument)


STDMETHODIMP CTIADoc::XDual_TIA_Doc::get_Name(BSTR FAR* pbstrName)
{
	METHOD_PROLOGUE(CTIADoc, Dual_TIA_Doc)
	TRY_DUAL(IID_ITIADocument)
	{
		(*pbstrName)= pThis->GetName();
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIADoc::XDual_TIA_Doc::get_FullPathName(BSTR FAR* pbstrFullPathName)
{
	METHOD_PROLOGUE(CTIADoc, Dual_TIA_Doc)
	TRY_DUAL(IID_ITIADocument)
	{
		(*pbstrFullPathName)= pThis->GetFullPathName();
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIADoc::XDual_TIA_Doc::get_Modified(long FAR* pnModified)
{
	METHOD_PROLOGUE(CTIADoc, Dual_TIA_Doc)
	TRY_DUAL(IID_ITIADocument)
	{
		(*pnModified)= pThis->GetModified();
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIADoc::XDual_TIA_Doc::put_Modified(long nModified)
{
	METHOD_PROLOGUE(CTIADoc, Dual_TIA_Doc)
	TRY_DUAL(IID_ITIADocument)
	{
		pThis->SetModified(nModified);
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIADoc::XDual_TIA_Doc::get_Visible(long FAR* pnVisible)
{
	METHOD_PROLOGUE(CTIADoc, Dual_TIA_Doc)
	TRY_DUAL(IID_ITIADocument)
	{
		(*pnVisible)= pThis->GetVisible();
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIADoc::XDual_TIA_Doc::put_Visible(long nVisible)
{
	METHOD_PROLOGUE(CTIADoc, Dual_TIA_Doc)
	TRY_DUAL(IID_ITIADocument)
	{
		pThis->SetVisible(nVisible);
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIADoc::XDual_TIA_Doc::Save(long FAR* pnResult)
{
	METHOD_PROLOGUE(CTIADoc, Dual_TIA_Doc)
	TRY_DUAL(IID_ITIADocument)
	{
		(*pnResult)= pThis->Save();
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIADoc::XDual_TIA_Doc::SaveAs(BSTR bstrFullPathName, long FAR* pnResult)
{
	METHOD_PROLOGUE(CTIADoc, Dual_TIA_Doc)
	TRY_DUAL(IID_ITIADocument)
	{
		CString cstr(bstrFullPathName);
		(*pnResult)= pThis->SaveAs(cstr);
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIADoc::XDual_TIA_Doc::AddColoredRect(long nLeft, long nTop, long nWidth, long nHeight, long nRGB, ITIARectangle FAR* FAR* ppRect)
{
	METHOD_PROLOGUE(CTIADoc, Dual_TIA_Doc)
	TRY_DUAL(IID_ITIADocument)
	{
		HRESULT hr= E_FAIL;
		LPDISPATCH pDisp= pThis->AddColoredRect(nLeft, nTop, nWidth, nHeight, nRGB);

		if (NULL!=pDisp)
		{
			hr= pDisp->QueryInterface(IID_ITIARectangle, (void **) ppRect);

			pDisp->Release();
			pDisp= NULL;
		}

		if (FAILED(hr))
		{
			AfxThrowOleException(hr);
		}

		return NOERROR;
	}
	CATCH_ALL_DUAL
}
