//	*****************************************************************************
//
//	Copyright (c) 1998 by David Cole -- All rights reserved.
//
//	Filename:	TIADOC.H
//
//	Author:		David P. Cole
//
//	Comments:
//
//				This source file may be used in any software for any purpose
//				as long as this notice and the copyright information remain
//				intact.
//
//				Send comments to:
//					David Cole
//					P.O. Box 2813
//					Seattle, WA 98111-2813
//					DLRdave@aol.com
//					http://members.aol.com/DLRdave
//
//	This header block generated by Commentizer v1.0
//
//	*****************************************************************************
//
//	Purpose:
//
//		Declaration of the CTIADoc object.
//
//	History:
//
//		Initial Version - Released with TIA 1.0.0001, November, 1998.
//
//	*****************************************************************************

// TIADoc.h : interface of the CTIADoc class
//
/////////////////////////////////////////////////////////////////////////////

#include "MFCDual.h"	//	DUAL_INTERFACE_SUPPORT
#include "TiaTLB.h"		//	ITIA* interfaces

class CTIASrvrItem;
class TIA_ColoredRectangle;

#define IsClosed()		(m_dwAutomationFlags & AF_OBJECT_DELETED)
#define MarkAsClosed()	(m_dwAutomationFlags |= AF_OBJECT_DELETED)

typedef enum tagTIAAutomationFlags {

	AF_NONE=			0x00000000L,
	AF_OBJECT_DELETED=	0x00000001L,
	AF_INVALID=			0xFFFFFFFFL

} TIAAutomationFlags;


class CTIADoc : public COleServerDoc
{
protected: // create from serialization only
	CTIADoc();
	DECLARE_DYNCREATE(CTIADoc)

// Attributes
public:
	CTIASrvrItem* GetEmbeddedItem()
		{ return (CTIASrvrItem*)COleServerDoc::GetEmbeddedItem(); }

// Operations
public:
	HRESULT AddTIARect(const CRect &r, const COLORREF &rgb, TIA_ColoredRectangle **ppRect= NULL);
	HRESULT DeleteTIAItems(void);
	HRESULT GetTIAItemList(CObList **ppList);
	HRESULT Changed(const CRect &r);
	HRESULT ForceClose(void);


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTIADoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual void OnFinalRelease();
	virtual void DeleteContents();
	protected:
	virtual COleServerItem* OnGetEmbeddedItem();
	//}}AFX_VIRTUAL

public:
	virtual void OnIdle();
	virtual BOOL DoSave(LPCTSTR lpszPathName, BOOL bReplace = TRUE);

// Implementation
public:
	virtual ~CTIADoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CTIADoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

	// Generated OLE dispatch map functions
	//{{AFX_DISPATCH(CTIADoc)
	afx_msg BSTR GetName();
	afx_msg BSTR GetFullPathName();
	afx_msg long GetModified();
	afx_msg void SetModified(long nNewValue);
	afx_msg long GetVisible();
	afx_msg void SetVisible(long nNewValue);
	afx_msg long Save();
	afx_msg long SaveAs(LPCTSTR szFullPathName);
	afx_msg LPDISPATCH AddColoredRect(long nTop, long nLeft, long nHeight, long nWidth, long nRGB);
	//}}AFX_DISPATCH
	DECLARE_DISPATCH_MAP()
	DECLARE_INTERFACE_MAP()

// *****************************************************************************
//  DUAL_INTERFACE_SUPPORT
	BEGIN_DUAL_INTERFACE_PART(Dual_TIA_Doc, ITIADocument)
		//	Copy and paste ITIADocument methods here from TiaTLB.h
		//	after it has been generated for the first time...
		//	Don't forget to remove the "PURE" or "=0" designation.
		//	We do need to make this a concrete class...

	    STDMETHOD(get_Name)(THIS_ BSTR FAR* pbstrName);
	    STDMETHOD(get_FullPathName)(THIS_ BSTR FAR* pbstrFullPathName);
	    STDMETHOD(get_Modified)(THIS_ long FAR* pnModified);
	    STDMETHOD(put_Modified)(THIS_ long nModified);
	    STDMETHOD(get_Visible)(THIS_ long FAR* pnVisible);
	    STDMETHOD(put_Visible)(THIS_ long nVisible);
	    STDMETHOD(Save)(THIS_ long FAR* pnResult);
	    STDMETHOD(SaveAs)(THIS_ BSTR bstrFullPathName, long FAR* pnResult);
	    STDMETHOD(AddColoredRect)(THIS_ long nLeft, long nTop, long nWidth, long nHeight, long nRGB, ITIARectangle FAR* FAR* ppRect);

	END_DUAL_INTERFACE_PART(Dual_TIA_Doc)
    DECLARE_DUAL_ERRORINFO()

private:
	CObList m_TIADocItems;
	BOOL m_bChangedWhileDeferring;

	//	Flags to guard against illegal Automation operations when doc
	//	is in certain states. Automation methods which should not be
	//	carried out with doc in those states should check these flags.
	DWORD m_dwAutomationFlags;
};

/////////////////////////////////////////////////////////////////////////////
