//	*****************************************************************************
//
//	Copyright (c) 1998 by David Cole -- All rights reserved.
//
//	Filename:	TIAITEMS.CPP
//
//	Author:		David P. Cole
//
//	Comments:
//
//				This source file may be used in any software for any purpose
//				as long as this notice and the copyright information remain
//				intact.
//
//				Send comments to:
//					David Cole
//					P.O. Box 2813
//					Seattle, WA 98111-2813
//					DLRdave@aol.com
//					http://members.aol.com/DLRdave
//
//	This header block generated by Commentizer v1.0
//
//	*****************************************************************************
//
//	Purpose:
//
//		Implementation of TIA_DocItem and TIA_ColoredRectangle.
//
//	History:
//
//		Initial Version - Released with TIA 1.0.0001, November, 1998.
//
//	*****************************************************************************

#include "stdafx.h"
#include "TIAItems.h"	//	TIA_DocItem, TIA_ColoredRectangle
#include "TRectAut.h"	//	CTIARectAut, the automation interface to TIA_ColoredRectangle


//	***************************************************************************
//	TIA_DocItem - base class of all items in the document item list

IMPLEMENT_SERIAL(TIA_DocItem, CObject, 1);

TIA_DocItem::TIA_DocItem()
{
	m_pDoc= NULL;
}

TIA_DocItem::~TIA_DocItem()
{
	m_pDoc= NULL;
}

HRESULT TIA_DocItem::SetDoc(CTIADoc *pDoc)
{
	if (NULL!=m_pDoc)
	{
		//	Already have a document, can't set again:
		return E_UNEXPECTED;
	}

	m_pDoc= pDoc;
	return NOERROR;
}

CTIADoc *TIA_DocItem::GetDoc(void)
{
	return m_pDoc;
}


//	***************************************************************************
//	TIA_ColoredRectangle - the only real item subclass

IMPLEMENT_SERIAL(TIA_ColoredRectangle, TIA_DocItem, 1);

TIA_ColoredRectangle::TIA_ColoredRectangle()
{
	m_rect.SetRect(-1,-1,-1,-1);
	m_rgb= RGB(0, 0, 0);
	m_pAut= NULL;
}

TIA_ColoredRectangle::~TIA_ColoredRectangle()
{
	if (NULL!=m_pAut)
	{
		m_pAut->ForgetImplementationObject(this);
		m_pAut= NULL;
	}
}

HRESULT TIA_ColoredRectangle::SetRect(const CRect &r)
{
	m_rect= r;
	return NOERROR;
}

HRESULT TIA_ColoredRectangle::GetRect(CRect &r) const
{
	r= m_rect;
	return NOERROR;
}

HRESULT TIA_ColoredRectangle::SetColor(const COLORREF &rgb)
{
	m_rgb= rgb;
	return NOERROR;
}

HRESULT TIA_ColoredRectangle::GetColor(COLORREF &rgb) const
{
	rgb= m_rgb;
	return NOERROR;
}

void TIA_ColoredRectangle::Serialize(CArchive &ar)
{
	if (ar.IsStoring())
	{
		ar << m_rect;
		ar << m_rgb;
	}
	else
	{
		ar >> m_rect;
		ar >> m_rgb;
	}
}

LPDISPATCH TIA_ColoredRectangle::GetAutomationObject(void)
{
	HRESULT hr= NOERROR;
	LPDISPATCH pDisp= NULL;

	//	Destroying an automation object "related" to an
	//	implementation object depends on two things:
	//
	//	<1> If m_pAut still exists at our destructor time,
	//		we tell it to forget about us by calling
	//		ForgetImplementationObject.
	//	<2> If m_pAut receives a final release while it
	//		"knows" about us, it calls us to ForgetAutomationObject.
	//
	//	"Whoever goes away first lets the other one know."

	if (NULL==m_pAut)
	{
		//	Create a new object and return a reference to it:
		m_pAut= new CTIARectAut;
		if (NULL!=m_pAut)
		{
			m_pAut->InitRectAut(this);

			//	CCmdTargets are created with a refcount, no need to AddRef or QI:
			pDisp= m_pAut->GetIDispatch(FALSE);
		}
	}
	else
	{
		//	Add a reference to the existing object:
		hr= m_pAut->ExternalQueryInterface(&IID_IDispatch, (void **) &pDisp);
		ASSERT(NOERROR==hr);
	}

	return pDisp;
}

void TIA_ColoredRectangle::ForgetAutomationObject(void)
{
	//	Destroying an automation object "related" to an
	//	implementation object depends on two things:
	//
	//	<1> If m_pAut still exists at our destructor time,
	//		we tell it to forget about us by calling
	//		ForgetImplementationObject.
	//	<2> If m_pAut receives a final release while it
	//		"knows" about us, it calls us to ForgetAutomationObject.
	//
	//	"Whoever goes away first lets the other one know."

	m_pAut= NULL;
}
