Attribute VB_Name = "TIATest"
'//  *****************************************************************************
'//
'//  Copyright (c) 1998 by David Cole -- All rights reserved.
'//
'//  Filename:   TIATEST.BAS
'//
'//  Author:     David P. Cole
'//
'//  Comments:
'//
'//              This source file may be used in any software for any purpose
'//              as long as this notice and the copyright information remain
'//              intact.
'//
'//              Send comments to:
'//                  David Cole
'//                  P.O. Box 2813
'//                  Seattle, WA 98111-2813
'//                  DLRdave@aol.com
'//                  http://members.aol.com/DLRdave
'//
'//  This header block generated by Commentizer v1.0
'//
'//  *****************************************************************************
'//
'//  Purpose:
'//
'//      Implement macros:
'//         TIATestObject - test IDispatch Automation capabilities
'//         TIATestDual - test dual Automation capabilities
'//         TIATest - do timing comparisons of Dim as Object
'//             vs. Dim as TIA.*
'//         SampleSequenceDiagram - run through code as illustrated
'//             in "Thin COM Wrapper" sequence diagram
'//
'//         Other supporting macros and functions for Form interface.
'//
'//  History:
'//
'//      Initial Version - Released with TIA 1.0.0001, November, 1998.
'//
'//  *****************************************************************************

Option Explicit

' Set to True for verbose debug output (see TIADebugPrint)
#Const TIA_VERBOSE = False

Const MAX_X = 5
Const MAX_Y = 5
Const MAX_CYCLES = 12

Public m_oApp As Object
Public m_oDoc As Object
Public m_oAppDual As TIA.Application
Public m_oDocDual As TIA.Document
Public m_dBegin As Double
Public m_dEnd As Double


Public Sub TIADebugPrint(ByRef sz As String)
#If TIA_VERBOSE Then
    Debug.Print sz
#Else
    ' Nothing!
#End If
End Sub


Public Function TIAMakeColor(ByVal nR As Long, ByVal nG As Long, ByVal nB As Long) As Long
    If nR < 0 Or nR > 255 Or nG < 0 Or nG > 255 Or nB < 0 Or nB > 255 Then
        Debug.Print "* BOGUS color component value passed to TIAMakeColor..."
        Err.Raise 5001
    End If
    TIAMakeColor = (nB * 256 * 256) + (nG * 256) + nR
End Function


Public Sub ParseTextForSevenNumbers(ByVal szText As String, ByRef n1 As Long, ByRef n2 As Long, ByRef n3 As Long, ByRef n4 As Long, ByRef n5 As Long, ByRef n6 As Long, ByRef n7 As Long)
    Dim nPos As Long

    n1 = 0
    n2 = 0
    n3 = 0
    n4 = 0
    n5 = 0
    n6 = 0
    n7 = 0

    nPos = InStr(szText, ",")
    If (nPos > 0) Then
        n1 = Val(Left(szText, nPos - 1))
        szText = Right(szText, Len(szText) - nPos)
    Else
        n1 = Val(szText)
    End If

    nPos = InStr(szText, ",")
    If (nPos > 0) Then
        n2 = Val(Left(szText, nPos - 1))
        szText = Right(szText, Len(szText) - nPos)
    Else
        n2 = Val(szText)
    End If

    nPos = InStr(szText, ",")
    If (nPos > 0) Then
        n3 = Val(Left(szText, nPos - 1))
        szText = Right(szText, Len(szText) - nPos)
    Else
        n3 = Val(szText)
    End If

    nPos = InStr(szText, ",")
    If (nPos > 0) Then
        n4 = Val(Left(szText, nPos - 1))
        szText = Right(szText, Len(szText) - nPos)
    Else
        n4 = Val(szText)
    End If

    nPos = InStr(szText, ",")
    If (nPos > 0) Then
        n5 = Val(Left(szText, nPos - 1))
        szText = Right(szText, Len(szText) - nPos)
    Else
        n5 = Val(szText)
    End If

    nPos = InStr(szText, ",")
    If (nPos > 0) Then
        n6 = Val(Left(szText, nPos - 1))
        szText = Right(szText, Len(szText) - nPos)
    Else
        n6 = Val(szText)
    End If

    nPos = InStr(szText, ",")
    If (nPos > 0) Then
        n7 = Val(Left(szText, nPos - 1))
        szText = Right(szText, Len(szText) - nPos)
    Else
        n7 = Val(szText)
    End If
End Sub


Public Sub StartTiming(ByRef sz As String)
    ' Use StartTiming and EndTiming calls to bracket
    ' some piece of code about which you want to know
    ' how long it takes... Assumption is made if m_dBegin
    ' is > m_dEnd that midnight was passed in the middle...
    ' See F1 help on Timer to understand why.
    ' Warning is spit out if that seems to be the case.

    Debug.Print "* " & sz & " - StartTiming called at " & Time

    m_dBegin = Timer
End Sub

Public Sub EndTiming(ByRef sz As String)
    ' Use StartTiming and EndTiming calls to bracket
    ' some piece of code about which you want to know
    ' how long it takes... Assumption is made if m_dBegin
    ' is > m_dEnd that midnight was passed in the middle...
    ' See F1 help on Timer to understand why.
    ' Warning is spit out if that seems to be the case.

    Dim dElapsed As Double

    m_dEnd = Timer

    Debug.Print "* " & sz & " - EndTiming called at " & Time

    dElapsed = m_dEnd - m_dBegin
    If (dElapsed < 0) Then
        ' Must have past midnight since StartTiming, adjust accordingly:
        dElapsed = (86400 - m_dBegin) + m_dEnd
    End If
    Debug.Print "* Elapsed seconds:"
    Debug.Print "* ----> " & dElapsed

    If (m_dBegin > m_dEnd) Then
        Debug.Print "* WARNING! ""'Round Midnight"" assumption made in EndTiming..."
    End If

    Debug.Print "*"
End Sub


Public Sub TIATestObject()
    Dim oDoc As Object
    Dim i As Long
    Dim nCount As Long

    StartTiming "TIATestObject"

    Set m_oApp = CreateObject("TIA.Application")
    TestAppMethodsObject m_oApp

    nCount = m_oApp.GetDocumentCount
    For i = 1 To nCount
        Set oDoc = m_oApp.GetDocument(i)
        TestDocMethodsObject oDoc
    Next i
    m_oApp.RemoveDocument m_oDoc
    Set m_oDoc = Nothing
    Set m_oApp = Nothing

    EndTiming "TIATestObject"
End Sub

Public Sub TestAppMethodsObject(ByRef oApp As Object)
    TIADebugPrint "*********************************************************"
    TIADebugPrint "* TestAppMethodsObject *"
    TIADebugPrint "*"
    TIADebugPrint "*             Name: " & oApp.Name
    TIADebugPrint "*     FullPathName: " & oApp.FullPathName
    TIADebugPrint "*          Version: " & oApp.Version
    TIADebugPrint "*      CommandLine: " & oApp.CommandLine
    TIADebugPrint "*          Visible: " & oApp.Visible
    Set m_oDoc = oApp.AddDocument("")
    TIADebugPrint "*      AddDocument: " & m_oDoc
    TIADebugPrint "* GetDocumentCount: " & oApp.GetDocumentCount
End Sub

Public Sub TestDocMethodsObject(ByRef oDoc As Object)
    TIADebugPrint "*********************************************************"
    TIADebugPrint "* BEGIN TestDocMethodsObject *"
    TIADebugPrint "*"
    TIADebugPrint "*             Name: " & oDoc.Name
    TIADebugPrint "*     FullPathName: " & oDoc.FullPathName
    TIADebugPrint "*         Modified: " & oDoc.Modified
    TIADebugPrint "*          Visible: " & oDoc.Visible
    TIADebugPrint "*           SaveAs: " & oDoc.SaveAs("TestDocMethods_" & oDoc.Name & ".tia")

    AddSomeRectanglesObject oDoc

    TIADebugPrint "*         Modified: " & oDoc.Modified
    TIADebugPrint "*             Save: " & oDoc.Save
    TIADebugPrint "* Setting Visible to True..."
    oDoc.Visible = True
    TIADebugPrint "*         Modified: " & oDoc.Modified
    TIADebugPrint "*     FullPathName: " & oDoc.FullPathName
    TIADebugPrint "*"
    TIADebugPrint "* END TestDocMethodsObject *"
    TIADebugPrint "*********************************************************"
End Sub

Public Sub AddSomeRectanglesObject(ByRef oDoc As Object)
    Dim oItem As Object
    Dim x As Long
    Dim y As Long
    Dim l As Long
    Dim T As Long
    Dim w As Long
    Dim h As Long
    Dim rgb As Long

    TIADebugPrint "*********************************************************"
    TIADebugPrint "* BEGIN AddSomeRectanglesObject *"
    TIADebugPrint "*"

    For y = 1 To MAX_Y
        TIADebugPrint "y: " & y
        For x = 1 To MAX_X
            Set oItem = oDoc.AddColoredRect(x * 10, y * 10, 9, 9, TIAMakeColor(0, 255, 0))
            oItem.GetRectangle l, T, w, h
            rgb = oItem.Color
            If (x = y) Then
                oItem.SetRectangle l - 1, T - 1, w + 2, h + 2
                oItem.Color = 0
                TIADebugPrint oItem.Name & " " & l & ", " & T & ", " & w & ", " & h & ", " & rgb
            End If
        Next x
    Next y

    TIADebugPrint "*"
    TIADebugPrint "* END AddSomeRectanglesObject *"
    TIADebugPrint "*********************************************************"
End Sub

Public Sub TIATestDual()
    Dim oDoc As TIA.Document
    Dim i As Long
    Dim nCount As Long

    StartTiming "TIATestDual"

    Set m_oAppDual = CreateObject("TIA.Application")
    TestAppMethodsDual m_oAppDual

    nCount = m_oAppDual.GetDocumentCount
    For i = 1 To nCount
        Set oDoc = m_oAppDual.GetDocument(i)
        TestDocMethodsDual oDoc
    Next i
    m_oAppDual.RemoveDocument m_oDocDual
    Set m_oDocDual = Nothing
    Set m_oAppDual = Nothing

    EndTiming "TIATestDual"
End Sub

Public Sub TestAppMethodsDual(ByRef oApp As TIA.Application)
    TIADebugPrint "*********************************************************"
    TIADebugPrint "* TestAppMethodsDual *"
    TIADebugPrint "*"
    TIADebugPrint "*             Name: " & oApp.Name
    TIADebugPrint "*     FullPathName: " & oApp.FullPathName
    TIADebugPrint "*          Version: " & oApp.Version
    TIADebugPrint "*      CommandLine: " & oApp.CommandLine
    TIADebugPrint "*          Visible: " & oApp.Visible
    Set m_oDocDual = oApp.AddDocument("")
    TIADebugPrint "*      AddDocument: " & m_oDocDual
    TIADebugPrint "* GetDocumentCount: " & oApp.GetDocumentCount
End Sub

Public Sub TestDocMethodsDual(ByRef oDoc As TIA.Document)
    TIADebugPrint "*********************************************************"
    TIADebugPrint "* BEGIN TestDocMethodsDual *"
    TIADebugPrint "*"
    TIADebugPrint "*             Name: " & oDoc.Name
    TIADebugPrint "*     FullPathName: " & oDoc.FullPathName
    TIADebugPrint "*         Modified: " & oDoc.Modified
    TIADebugPrint "*          Visible: " & oDoc.Visible
    TIADebugPrint "*           SaveAs: " & oDoc.SaveAs("TestDocMethods_" & oDoc.Name & ".tia")

    AddSomeRectanglesDual oDoc

    TIADebugPrint "*         Modified: " & oDoc.Modified
    TIADebugPrint "*             Save: " & oDoc.Save
    TIADebugPrint "* Setting Visible to True..."
    oDoc.Visible = True
    TIADebugPrint "*         Modified: " & oDoc.Modified
    TIADebugPrint "*     FullPathName: " & oDoc.FullPathName
    TIADebugPrint "*"
    TIADebugPrint "* END TestDocMethodsDual *"
    TIADebugPrint "*********************************************************"
End Sub

Public Sub AddSomeRectanglesDual(ByRef oDoc As TIA.Document)
    Dim oItem As TIA.Rectangle
    Dim x As Long
    Dim y As Long
    Dim l As Long
    Dim T As Long
    Dim w As Long
    Dim h As Long
    Dim rgb As Long

    TIADebugPrint "*********************************************************"
    TIADebugPrint "* BEGIN AddSomeRectanglesDual *"
    TIADebugPrint "*"

    For y = 1 To MAX_Y
        TIADebugPrint "y: " & y
        For x = 1 To MAX_X
            Set oItem = oDoc.AddColoredRect(x * 10, y * 10, 9, 9, TIAMakeColor(0, 255, 0))
            oItem.GetRectangle l, T, w, h
            rgb = oItem.Color
            If (x = y) Then
                oItem.SetRectangle l - 1, T - 1, w + 2, h + 2
                oItem.Color = 0
                TIADebugPrint oItem.Name & " " & l & ", " & T & ", " & w & ", " & h & ", " & rgb
            End If
        Next x
    Next y

    TIADebugPrint "*"
    TIADebugPrint "* END AddSomeRectanglesDual *"
    TIADebugPrint "*********************************************************"
End Sub

Public Sub TIATest()
    Dim i As Long
    Dim oApp As Object
    Dim oAppDual As TIA.Application
    Dim bVisible As Long

    ' Make sure app is launched before we do test suites:
    ' (so launching doesn't affect our timing measurements)
    Set oApp = CreateObject("TIA.Application")
    Set oAppDual = oApp

    ' Save visible state and make visible:
    bVisible = oAppDual.Visible
    oAppDual.Visible = True

    Debug.Print "*********************************************************"
    Debug.Print "* Running TIA tests..."
    Debug.Print "* " & Time
    Debug.Print "* " & oAppDual.FullPathName
    Debug.Print "*"

    ' Run the timing/testing suites:
    For i = 1 To MAX_CYCLES
        Debug.Print "*********************************************************"
        Debug.Print "* Run number: " & i
        Debug.Print "*****************"
        TIATestObject
        TIATestDual
    Next i

    Debug.Print "*"
    Debug.Print "* " & Time
    Debug.Print "* Done with TIA tests..."
    Debug.Print "*********************************************************"

    ' Restore initial visible state:
    oAppDual.Visible = bVisible

    ' When going out of scope, if we leave the app visible
    ' when it was initially invisible, the visibility has a
    ' "User Lock" inside TIA and so the app will not shut down
    ' when oApp and oAppDual go out of scope. If, on the other
    ' hand, we make the app invisible, then there is no
    ' "User Lock" on it, and as long as we are the only
    ' outstanding object references to TIA, it should shut
    ' down when we go out of scope.
End Sub


Public Sub SampleSequenceDiagram()
    Dim oApp As TIA.Application
    Dim oDoc As TIA.Document
    Dim oRect As TIA.Rectangle

    Set oApp = CreateObject("TIA.Application")
    Set oDoc = oApp.GetDocument(1)
    Set oRect = oDoc.AddColoredRect(10, 10, 20, 20, 0)
    oApp.RemoveDocument oDoc
    Set oDoc = Nothing

    ' After this point oRect will not be able to communicate
    ' with its underlying C++ implementation object...
    ' The underlying object gets destroyed when the document
    ' gets destroyed in the "Set oDoc = Nothing" call.

    Set oRect = Nothing
End Sub
