//	*****************************************************************************
//
//	Copyright (c) 1998 by David Cole -- All rights reserved.
//
//	Filename:	TIAVIEW.CPP
//
//	Author:		David P. Cole
//
//	Comments:
//
//				This source file may be used in any software for any purpose
//				as long as this notice and the copyright information remain
//				intact.
//
//				Send comments to:
//					David Cole
//					P.O. Box 2813
//					Seattle, WA 98111-2813
//					DLRdave@aol.com
//					http://members.aol.com/DLRdave
//
//	This header block generated by Commentizer v1.0
//
//	*****************************************************************************
//
//	Purpose:
//
//		Implementation of CTIAView.
//
//	History:
//
//		Initial Version - Released with TIA 1.0.0001, November, 1998.
//
//	*****************************************************************************

// TIAView.cpp : implementation of the CTIAView class
//

#include "stdafx.h"
#include "TIA.h"

#include "TIADoc.h"
#include "TIAView.h"

#include "TIAItems.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTIAView

IMPLEMENT_DYNCREATE(CTIAView, CView)

BEGIN_MESSAGE_MAP(CTIAView, CView)
	//{{AFX_MSG_MAP(CTIAView)
	ON_COMMAND(ID_CANCEL_EDIT_SRVR, OnCancelEditSrvr)
	ON_WM_LBUTTONUP()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTIAView construction/destruction

COLORREF _rgb[]= {
	RGB(255, 0, 0),
	RGB(0, 0, 255),
	RGB(255, 255, 0),
	RGB(127, 127, 127),
};

long _nIndex= sizeof(_rgb)/sizeof(_rgb[0]) - 1;

CTIAView::CTIAView()
{
	//	Cycle through array of colors:
	_nIndex++;
	if (sizeof(_rgb)/sizeof(_rgb[0])==_nIndex) _nIndex= 0;
	m_rgb= _rgb[_nIndex];

	m_nWidth= 10;
	m_nHeight= 10;
}

CTIAView::~CTIAView()
{
}

BOOL CTIAView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTIAView drawing

void CTIAView::OnDraw(CDC* pDC)
{
	CTIADoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CObList *pList= NULL;
	POSITION pos= NULL;
	TIA_ColoredRectangle *pItem= NULL;
	CRect r;
	COLORREF rgb= RGB(0, 0, 0);
	CBrush brush;

	//	Draw the document items:
	if (SUCCEEDED(pDoc->GetTIAItemList(&pList)))
	{
		pos= pList->GetHeadPosition();
		if (NULL!=pos)
		{
			pItem= (TIA_ColoredRectangle *) pList->GetNext(pos);
		}

		while (NULL!=pItem)
		{
			//	We only know how to draw the TIA_ColoredRectangle
			//	items in the list...
			if (pItem->IsKindOf(RUNTIME_CLASS(TIA_ColoredRectangle)))
			{
				pItem->GetRect(r);
				pItem->GetColor(rgb);

				brush.CreateSolidBrush(rgb);
				pDC->FillRect(r, &brush);
				brush.DeleteObject();
			}

			if (NULL==pos)
			{
				pItem= NULL;
			}
			else
			{
				pItem= (TIA_ColoredRectangle *) pList->GetNext(pos);
			}
		}
	}

	//	Frame the view with a line representing its creation color:
	brush.CreateSolidBrush(m_rgb);
	GetClientRect(&r);

	r.InflateRect(-2, -2);
	r.right -= 1;
	r.bottom -= 1;
	pDC->FrameRect(r, &brush);
	r.InflateRect(1, 1);
	pDC->FrameRect(r, &brush);
	r.InflateRect(1, 1);
	pDC->FrameRect(r, &brush);

	brush.DeleteObject();
}

/////////////////////////////////////////////////////////////////////////////
// CTIAView printing

BOOL CTIAView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CTIAView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CTIAView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// OLE Server support

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the server (not the container) causes the deactivation.
void CTIAView::OnCancelEditSrvr()
{
	GetDocument()->OnDeactivateUI(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CTIAView diagnostics

#ifdef _DEBUG
void CTIAView::AssertValid() const
{
	CView::AssertValid();
}

void CTIAView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTIADoc* CTIAView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTIADoc)));
	return (CTIADoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTIAView message handlers

void CTIAView::OnLButtonUp(UINT /* nFlags */, CPoint point)
{
	CTIADoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->AddTIARect(
		CRect(point.x, point.y, point.x+m_nWidth, point.y+m_nHeight),
		m_rgb
		);
}

HRESULT CTIAView::SetColor(COLORREF &rgb)
{
	m_rgb= rgb;
	return NOERROR;
}
