//	*****************************************************************************
//
//	Copyright (c) 1998 by David Cole -- All rights reserved.
//
//	Filename:	TRECTAUT.CPP
//
//	Author:		David P. Cole
//
//	Comments:
//
//				This source file may be used in any software for any purpose
//				as long as this notice and the copyright information remain
//				intact.
//
//				Send comments to:
//					David Cole
//					P.O. Box 2813
//					Seattle, WA 98111-2813
//					DLRdave@aol.com
//					http://members.aol.com/DLRdave
//
//	This header block generated by Commentizer v1.0
//
//	*****************************************************************************
//
//	Purpose:
//
//		Implementation of CTIARectAut, the TIA.Rectangle
//		Automation object. A CTIARectAut is a "thin COM
//		wrapper" for a TIA_ColoredRectangle.
//
//	History:
//
//		Initial Version - Released with TIA 1.0.0001, November, 1998.
//
//	*****************************************************************************

// TRectAut.cpp : implementation file
//

#include "stdafx.h"
#include "tia.h"
#include "TRectAut.h"

#include "TIAItems.h"	//	for TIA_ColoredRectangle, the implementation object
#include "MFCUtils.h"	//	for MU_SetDeferringUpdates
#include "TIADoc.h"		//	for calling CTIADoc::Changed

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTIARectAut

IMPLEMENT_DYNCREATE(CTIARectAut, CCmdTarget)

CTIARectAut::CTIARectAut()
{
	EnableAutomation();

	m_pImp= NULL;

	// To keep the application running as long as an OLE automation 
	//	object is active, the constructor calls AfxOleLockApp.
	AfxOleLockApp();
}

CTIARectAut::~CTIARectAut()
{
	// To terminate the application when all objects created with
	// 	with OLE automation, the destructor calls AfxOleUnlockApp.
	AfxOleUnlockApp();
}


void CTIARectAut::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	if (NULL!=m_pImp)
	{
		m_pImp->ForgetAutomationObject();
		m_pImp= NULL;
	}

	CCmdTarget::OnFinalRelease();
}


BEGIN_MESSAGE_MAP(CTIARectAut, CCmdTarget)
	//{{AFX_MSG_MAP(CTIARectAut)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CTIARectAut, CCmdTarget)
	//{{AFX_DISPATCH_MAP(CTIARectAut)
	DISP_PROPERTY_EX(CTIARectAut, "Color", GetColor, SetColor, VT_I4)
	DISP_PROPERTY_EX(CTIARectAut, "Name", GetName, SetNotSupported, VT_BSTR)
	DISP_FUNCTION(CTIARectAut, "SetRectangle", SetRectangle, VT_EMPTY, VTS_I4 VTS_I4 VTS_I4 VTS_I4)
	DISP_FUNCTION(CTIARectAut, "GetRectangle", GetRectangle, VT_EMPTY, VTS_PI4 VTS_PI4 VTS_PI4 VTS_PI4)
	DISP_DEFVALUE(CTIARectAut, "Name")
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// *****************************************************************************
//	ClassWizard Leftovers:

// Note: we add support for IID_ITIARectAut to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {8857D282-6223-11D2-9B8C-44455354616F}
//static const IID IID_ITIARectAut =
//{ 0x8857d282, 0x6223, 0x11d2, { 0x9b, 0x8c, 0x44, 0x45, 0x53, 0x54, 0x61, 0x6f } };
// *****************************************************************************
//	CHANGED_GUID

BEGIN_INTERFACE_MAP(CTIARectAut, CCmdTarget)
	INTERFACE_PART(CTIARectAut, DIID_ITIARectangleDisp, Dispatch)		//	CHANGED_GUID
	INTERFACE_PART(CTIARectAut, IID_ITIARectangle, Dual_TIA_Rect)		//  DUAL_INTERFACE_SUPPORT
	DUAL_ERRORINFO_PART(CTIARectAut)									//  DUAL_INTERFACE_SUPPORT
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTIARectAut - implementation bridge

long CTIARectAut::InitRectAut(TIA_ColoredRectangle *pImp)
{
	ASSERT(NULL==m_pImp);		//	only init once...
	m_pImp= pImp;
	return 0L;	//	NOERROR
}

void CTIARectAut::ForgetImplementationObject(TIA_ColoredRectangle *pImp)
{
	ASSERT(pImp==m_pImp);
		//	only call forget with the one we knew...

	pImp;
		//	avoids warning about unreferenced formal parameter in Release build...

	m_pImp= NULL;
}

//	If implementation object isn't there (either it "went away"
//	or we were not properly initialized with it...) then throw
//	back an "unexpected" exception. Use this macro at the top
//	of every automation method before accessing m_pImp:

#define VerifyRectImplementationObject()			\
	if (NULL==m_pImp)								\
	{												\
		AfxThrowOleException(E_UNEXPECTED);			\
	}


/////////////////////////////////////////////////////////////////////////////
// CTIARectAut message handlers

long CTIARectAut::GetColor() 
{
	VerifyRectImplementationObject();

	COLORREF rgb;
	HRESULT hr= m_pImp->GetColor(rgb);
	if (FAILED(hr))
	{
		AfxThrowOleException(hr);
	}

	return (long) rgb;
}

void CTIARectAut::SetColor(long nNewValue) 
{
	VerifyRectImplementationObject();

	HRESULT hr= m_pImp->SetColor((COLORREF) nNewValue);
	if (FAILED(hr))
	{
		AfxThrowOleException(hr);
	}

	//	Now send a deferred change message to the item's document:
	CRect r;
	hr= m_pImp->GetRect(r);
	if (FAILED(hr))
	{
		AfxThrowOleException(hr);
	}

	ASSERT(NULL!=m_pImp->GetDoc());
		//	how did we get an implementation object without a document?

	MU_SetDeferringUpdates();
	m_pImp->GetDoc()->Changed(r);
}

void CTIARectAut::SetRectangle(long nLeft, long nTop, long nWidth, long nHeight) 
{
	VerifyRectImplementationObject();

	CRect r(nLeft, nTop, nLeft+nWidth, nTop+nHeight);
	HRESULT hr= m_pImp->SetRect(r);
	if (FAILED(hr))
	{
		AfxThrowOleException(hr);
	}

	ASSERT(NULL!=m_pImp->GetDoc());
		//	how did we get an implementation object without a document?

	MU_SetDeferringUpdates();
	m_pImp->GetDoc()->Changed(r);
}

void CTIARectAut::GetRectangle(long FAR* pnLeft, long FAR* pnTop, long FAR* pnWidth, long FAR* pnHeight) 
{
	VerifyRectImplementationObject();

	CRect r;
	HRESULT hr= m_pImp->GetRect(r);
	if (FAILED(hr))
	{
		AfxThrowOleException(hr);
	}

	(*pnLeft)= r.left;
	(*pnTop)= r.top;
	(*pnWidth)= r.Width();
	(*pnHeight)= r.Height();
}

BSTR CTIARectAut::GetName() 
{
	VerifyRectImplementationObject();

	CString strResult;
	strResult.Format( _T("TIA.Rectangle.0x%08lx"), ((long) m_pImp) );
	return strResult.AllocSysString();
}


// *****************************************************************************
//  DUAL_INTERFACE_SUPPORT

//	In addition to this bottom section of the file, there are also two extra
//	entries in the interface map up above. One for the dual interface and one
//	for ISupportErrorInfo.

//	Delegate standard IDispatch methods to MFC IDispatch implementation:
DELEGATE_DUAL_INTERFACE(CTIARectAut, Dual_TIA_Rect)

//	Implement ISupportErrorInfo for this dual interface:
IMPLEMENT_DUAL_ERRORINFO(CTIARectAut, IID_ITIARectangle)


STDMETHODIMP CTIARectAut::XDual_TIA_Rect::get_Name(BSTR FAR* pbstrName)
{
	METHOD_PROLOGUE(CTIARectAut, Dual_TIA_Rect)
	TRY_DUAL(IID_ITIARectangle)
	{
		(*pbstrName)= pThis->GetName();
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIARectAut::XDual_TIA_Rect::get_Color(long FAR* pnColor)
{
	METHOD_PROLOGUE(CTIARectAut, Dual_TIA_Rect)
	TRY_DUAL(IID_ITIARectangle)
	{
		(*pnColor)= pThis->GetColor();
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIARectAut::XDual_TIA_Rect::put_Color(long nColor)
{
	METHOD_PROLOGUE(CTIARectAut, Dual_TIA_Rect)
	TRY_DUAL(IID_ITIARectangle)
	{
		pThis->SetColor(nColor);
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIARectAut::XDual_TIA_Rect::SetRectangle(long nLeft, long nTop, long nWidth, long nHeight)
{
	METHOD_PROLOGUE(CTIARectAut, Dual_TIA_Rect)
	TRY_DUAL(IID_ITIARectangle)
	{
		pThis->SetRectangle(nLeft, nTop, nWidth, nHeight);
		return NOERROR;
	}
	CATCH_ALL_DUAL
}

STDMETHODIMP CTIARectAut::XDual_TIA_Rect::GetRectangle(long FAR* pnLeft, long FAR* pnTop, long FAR* pnWidth, long FAR* pnHeight)
{
	METHOD_PROLOGUE(CTIARectAut, Dual_TIA_Rect)
	TRY_DUAL(IID_ITIARectangle)
	{
		pThis->GetRectangle(pnLeft, pnTop, pnWidth, pnHeight);
		return NOERROR;
	}
	CATCH_ALL_DUAL
}
